#ifndef GF_BLOCK_H
#define GF_BLOCK_H

#include <render/gimage.h>
#include "gfprimtv.h"

extern const GString gfBlockID;

class GFBlock: public GFPrimitive {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: An orthogonal solid block.
    //
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GFBlock(const char*, const GVector3&, GFPrimitive* = (GFPrimitive*)gfNoFather);
    GFBlock(const char*, const GFBlock*, GFPrimitive* = (GFPrimitive*)gfNoFather);

    virtual bool isA(const GString& pString) const {
	return(pString == gfBlockID);
    }

    virtual const char* getDescription(void) const { return("Orthogonal solid block. Dimensions may be changed after creation."); }
    virtual const char* getClass(void) const { return(gfBlockID); }

    virtual void saveConstr(ostream&);

    void setDimensions(const GVector3&);
    const GVector3 getDimensions(void) const;

    void setImage(const char*);

    // overloaded ray-tracing interface:
    virtual const GSurface& getSurface(const GPoint3&);

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  private:
    virtual GFRenderable* getRenderable(void);
    static class GFBlockP* sBlock;
    GVector3 mDimensions;
    GSGIImage* mImage;
};

#endif
