#ifndef GF_CYLINDER_H
#define GF_CYLINDER_H

#include "gfprimtv.h"

extern const GString gfCylinderID;

class GFCylinder: public GFPrimitive {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A cylinder along y axis.
    // Radius and length may be specified.
    // The discretiziation in object space
    // may be specified arbitraryly and will multiple
    // be used.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GFCylinder(const char*, GFloat LENGTH, GFloat RADIUS, GFPrimitive* = (GFPrimitive*)gfNoFather);
    GFCylinder(const char*, const GFCylinder*, GFPrimitive* = (GFPrimitive*)gfNoFather);
    virtual ~GFCylinder(void);

    virtual bool isA(const GString& pString) const {
	return(pString == gfCylinderID);
    }
    virtual const char* getClass(void) const { return(gfCylinderID); }

    virtual const char* getDescription(void) const { return("Solid cylinder. The cylinder is centered around y axis and starts at x-z plane. Radius and length may be changed after creation."); }

    virtual void saveConstr(ostream&);

    void setRadius(GFloat);
    GFloat getRadius(void) const;

    void setLength(GFloat);
    GFloat getLength(void) const;
#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  private:
    virtual GFRenderable* getRenderable(void);
    
    virtual void resolutionChanged(void);
    GFloat mRadius;
    GFloat mLength;
    class GFCylinderP* mCylinder;
};

#endif
