#ifndef GF_LIGHT_H
#define GF_LIGHT_H

#include <render/grayligh.h>
#include "gftop.h"

extern const GString gfLightID;

class GFPointLight: public GRayPointLight, public GShadePointLight {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: Integration of ray-tracing and
    // shading compatible light source.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
};

class GFLight: public GFTop {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A primitive containing a light
    // source.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GFLight(const char*, GFPrimitive* = (GFPrimitive*)gfNoFather);
    virtual ~GFLight(void);
    
    virtual bool isA(const GString& pString) const {
	return(pString == gfLightID || GFTop::isA(pString));
    }

    virtual const char* getDescription(void) const { return("Encapsulation of a point light source. The point light is located at the origin of the primitive. Light sources are global. They don't need to be inserted into a scene. To pick and manipulate a light source just append a transparent primitive to it."); }
    virtual const char* getClass(void) const { return(gfLightID); }

    virtual void on(void) {
	mLight.on();
    }

    virtual void off(void) {
	mLight.off();
    }
  public:
    // assign to the global light source pointer
    // used by renderers:
    static void initialize(void) {
	gShadeLights = &sShadeList;
	gRayLights = &sRayList;
    }
  protected:
    virtual void modelingChanged(void);

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  protected:
    void setupLight(void);
  private:
    GFPointLight mLight;
    static GDLList<GRayLight> sRayList;
    static GDLList<GShadeLight> sShadeList;
};

#endif
