#ifndef GF_LOOKAT_H
#define GF_LOOKAT_H

#include "gftypes.h"
#include "gfscene.h"
#include "gfpixmap.h"

extern const GString gfLookatCameraID;

class GFLookatCamera
#ifdef GWITH_TCL
: public GTclInterface
#endif
{
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A class hiding the
    // lookat camera and containing both
    // shading and ray-tracing camera.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    // initial reference point is the WC origin:
    GFLookatCamera(const char*); 

    void shading(void);
    void rayTracing(GFloat);

    void setScene(GFScene& pScene) {
	mScene = &pScene;
    }
    GFScene& getScene(void) const {
	assert(mScene);
	return(*mScene);
    }

    void setPixmap(GFPixmap& pPixmap) {
	mPixmap = &pPixmap;
    }
    GFPixmap& getPixmap(void) {
	assert(mPixmap);
	return(*mPixmap);
    }

    void setZoom(GFloat);
    GFloat getZoom(void) const;

    void setViewpoint(const GPoint3&);
    const GVector3& getViewpoint(void) const;

    void setRefpoint(const GPoint3&);
    const GVector3& getRefpoint(void) const;

    const char* getDescription(void) const {
	return("A camera for shading and ray-tracing, basing on OpenGLs lookat camera model.");
    }

    void perspectivic(void) {
	mShader.perspectivic();
	mTracer.perspectivic();
    }
    void orthographic(void) {
	mShader.orthographic();
	mTracer.orthographic();
    }
  public:
    GFLookatShader& getShader(void) {
	return(mShader);
    }
    const GString& getName(void) const {
	return(mName);
    }

    static GFLookatCamera* getByName(const char*);

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  private:
    GFLookatShader mShader;
    GFLookatRayTracer mTracer; 

    GFScene* mScene;
    GFPixmap* mPixmap;
    const GString mName;

    static GDLList<GFLookatCamera> sList;
};

extern bool gfAutoUpdate;

#endif
