#ifndef GF_MANIPULATOR_H
#define GF_MANIPULATOR_H

#include <input/gupdate.h>
#include "gfdevice.h"
#include "gfpick.h"
#include "gflocat.h"
#include "gfmover.h"
#include "gfscene.h"
#include "gfbbox.h"

typedef enum GFManipulatorXform {
    GF_ROTATE, 
    GF_TRANSLATE, 
    GF_ROT_TRANS     
};

extern const GString gfManipulatorID;

class GFManipulator: public GFInputDevice 
#ifdef GWITH_TCL
, public GTclInterface
#endif
{
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A complex input class
    // for object and camera manipulations.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GFManipulator(const char*, GFLookatCamera&, GFScene&, GInputDevice* = (GInputDevice*)gNoInputDevice);
    virtual ~GFManipulator(void);

    void setObject(GFPrimitive*);

    GFPrimitive* getObject(void) const {
	return(mPrimitive);
    }

    const char* getDescription(void) const {
	return("An integrated input device for both object and camera manipulations. The manipulator will be triggered after object selections, deselections and manipulations.");
    }
  public:
    GFScene& getScene(void) {
	return(mScene);
    }

    GFLookatCamera& getCamera(void) {
	return(mRend);
    }

    virtual bool event(GEvent*);

    void translate(const GVector3&);

    void rotate(GFloat);

    void setXform(GFManipulatorXform pXform) {
	mXform = pXform;
    }
    GFManipulatorXform getXform(void) const { 
	return(mXform);
    }
    
    void fixCamera(void) {
	mChangeCam = false;
	mMov->off();
    }

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  private:
    GFPrimitive* mPrimitive;
    GFLookatCamera& mRend;
    GFScene& mScene;
    GFManipulatorXform mXform;
    bool mChangeCam;

    GFLocator* mLoc;
    GFMover* mMov;
    GUpdatePixmap* mUpd;
    GFBoundingBox* mBox;

    class PickCB: public GCallback {
      public:
	PickCB(GFManipulator& pManip): mManip(pManip) {}
	virtual void action(GInputDevice&);
      private:
	GFManipulator& mManip;
    };
    PickCB pickCB;

    class LocatorCB: public GCallback {
      public:
	LocatorCB(GFManipulator& pManip): mManip(pManip) {}
	virtual void action(GInputDevice&);
      private:
	GFManipulator& mManip;
    };
    LocatorCB locatorCB;

    class UpdateCB: public GCallback {
      public:
	UpdateCB(GFManipulator& pManip): mManip(pManip) {}
	virtual void action(GInputDevice&);
      private:
	GFManipulator& mManip;
    };
    UpdateCB updateCB;
};

#endif
