#ifndef GF_SCENE_H
#define GF_SCENE_H

#include "gfprimtv.h"

#ifdef GWITH_TCL
#include <util/gtcl.h>
#endif

extern const GString gfSceneID;

class GFScene
#ifdef GWITH_TCL
: public GTclInterface
#endif
{
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: The scene aggregate
    // that will be rendered by cameras.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GFScene(const char*);
    virtual ~GFScene(void);

    void insert(GFPrimitive& pPrim) {
	pPrim.computeBounds();
	mScene.append(&pPrim);
    }
    void remove(GFPrimitive& pPrim) {
	mScene.remove(&pPrim);
    }

    const char* getDescription(void) const {
	return("A dynamic aggregate of primitives which may be rendered by a camera.");
    }

    void setBackground(const GColorRGB& pColor) {
	mBackgr = pColor;
    }

    GAggregate<GFPrimitive>& getObjects(void) { 
	return(mScene); 
    } 

    // compute recursively bounding boxes:
    void computeBounds(void);

  public:
    void insertFdb(GFPrimitive& pPrim) {
	mFeedback.append(&pPrim);
    }
    void removeFdb(GFPrimitive& pPrim) {
	mFeedback.remove(&pPrim);
    }
    
    const GColorRGB& getBackground(void) const {
	return(mBackgr);
    }

    static GFScene* getByName(const char*);

    GAggregate<GFPrimitive>& getFeedback(void) {
	return(mFeedback);
    }

    const GString& getName(void) const {
	return(mName);
    }
    
#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
#endif
  private:
    GString mName;
    GDLList<GFPrimitive> mScene;
    GDLList<GFPrimitive> mFeedback;
    
    static GDLList<GFScene> sList;
    GColorRGB mBackgr;
};

#endif
