#ifndef GF_SWEEP_POLYGON_H
#define GF_SWEEP_POLYGON_H

#include "gfprimtv.h"
#include "gftypes.h"

class GFSweepPolygon: public GFPrimitive {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: Common stuff for sweep polygons.
    // 
    // INVARIANT: The polygon must be planar and convex.
    //
    // IMPLEMENTATION ISSUES: The polygon vertices have
    // to be specified in clock-wise direction. The polygon
    // is closed automatically.
    // CONSTANTS: 
  protected:
    GFSweepPolygon(const char* pName, GFPrimitive* pFather): GFPrimitive(pName, pFather) {}
  public:
    virtual void setLength(GFloat) = 0;
    GFloat getLength(void) const {
	return(mLength);
    }
    virtual void saveConstr(ostream&);
#ifdef GWITH_TCL
  public:
    virtual bool objectCMD(char*[]);
#endif
  protected:
    GFloat mLength;
};

extern const GString gfSweepXPolygonID;

class GFSweepXPolygon: public GFSweepPolygon {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A solid created by sweeping a
    // planar polygon in x direction. 
    // INVARIANT: The polygon must be planar and convex.
    //
    // IMPLEMENTATION ISSUES: The polygon vertices have
    // to be specified in clock-wise direction. The polygon
    // is closed automatically.
    // CONSTANTS: 
  public:
    GFSweepXPolygon(const char*, const GFPolygon2D&, GFloat, GFPrimitive* = (GFPrimitive*)gfNoFather);
    GFSweepXPolygon(const char*, const GFSweepXPolygon*, GFPrimitive* = (GFPrimitive*)gfNoFather);
    virtual ~GFSweepXPolygon(void);

    virtual void setLength(GFloat);

    virtual bool isA(const GString& pString) const {
	return(pString == gfSweepXPolygonID);
    }

    virtual const char* getDescription(void) const { return("X-axis sweeped planar polygon lying in y-z plane. The smallest point of the sweep must be (0, 0, 0). The length may be changed after creation."); }
    virtual const char* getClass(void) const { return(gfSweepXPolygonID); }

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
#endif
  private:
    virtual GFRenderable* getRenderable(void);
    class GFSweepXPolygonP* mSweepPolygon;
};

extern const GString gfSweepYPolygonID;

class GFSweepYPolygon: public GFSweepPolygon {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A solid created by sweeping a
    // planar polygon in y direction. 
    // INVARIANT: The polygon must be planar and convex.
    //
    // IMPLEMENTATION ISSUES: The polygon vertices have
    // to be specified in clock-wise direction. The polygon
    // is closed automatically.
    // CONSTANTS: 
  public:
    GFSweepYPolygon(const char*, const GFPolygon2D&, GFloat, GFPrimitive* = (GFPrimitive*)gfNoFather);
    GFSweepYPolygon(const char*, const GFSweepYPolygon*, GFPrimitive* = (GFPrimitive*)gfNoFather);
    virtual ~GFSweepYPolygon(void);

    virtual void setLength(GFloat);

    virtual bool isA(const GString& pString) const {
	return(pString == gfSweepYPolygonID);
    }

    virtual const char* getDescription(void) const { return("Y-axis sweeped planar polygon lying in x-z plane. The smallest point of the sweep must be (0, 0, 0). The length may be changed after creation."); }
    virtual const char* getClass(void) const { return(gfSweepYPolygonID); }

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
#endif
  private:
    virtual GFRenderable* getRenderable(void);
    class GFSweepYPolygonP* mSweepPolygon;
};

extern const GString gfSweepZPolygonID;

class GFSweepZPolygon: public GFSweepPolygon {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: A solid created by sweeping a
    // planar polygon in z direction. 
    // INVARIANT: The polygon must be planar and convex.
    //
    // IMPLEMENTATION ISSUES: The polygon vertices have
    // to be specified in clock-wise direction. The polygon
    // is closed automatically.
    // CONSTANTS: 
  public:
    GFSweepZPolygon(const char*, const GFPolygon2D&, GFloat, GFPrimitive* = (GFPrimitive*)gfNoFather);
    GFSweepZPolygon(const char*, const GFSweepZPolygon*, GFPrimitive* = (GFPrimitive*)gfNoFather);
    virtual ~GFSweepZPolygon(void);

    virtual void setLength(GFloat);

    virtual bool isA(const GString& pString) const {
	return(pString == gfSweepZPolygonID);
    }

    virtual const char* getDescription(void) const { return("Z-axis sweeped planar polygon lying in x-y plane. The smallest point of the sweep must be (0, 0, 0). The length may be changed after creation."); }
    virtual const char* getClass(void) const { return(gfSweepZPolygonID); }

#ifdef GWITH_TCL
  public:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
#endif
  private:
    virtual GFRenderable* getRenderable(void);
    class GFSweepZPolygonP* mSweepPolygon;
};

#endif
