#ifndef GF_TCL_H
#define GF_TCL_H

#ifdef GWITH_TCL

#include "gfdevice.h"
#include "gfprimtv.h"
#include "gftop.h"

class GFTclCallback: public GCallback {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: Generic Tcl callback
    // procedure for input devices.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES: This callback may only be used
    // for GF input device because it performs a type cast
    // to GF input device w/o ony possibility to verify it.
    // CONSTANTS: 
  public:
    GFTclCallback(const char* pProc): mProc(pProc) {}
    
    virtual void action(GInputDevice&);
  private:
    GString mProc;
};

extern const GString gfGfmID;
extern const GString gfGfpID;

class GFTclPrimitive: public GFPrimitive, public GTclClass {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: Base class for Tcl extensible 
    // primitive classes.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  protected:
    GFTclPrimitive(const char *, GFPrimitive*, GTclClassDesc*, char*[]);

    virtual bool isA(const GString& pString) const {
	return(GTclClass::isA(pString));
    }

    virtual const char* getDescription(void) const { return(GTclClass::getDescription()); }
    virtual const char* getClass(void) const { return(GTclClass::getClass()); }

    virtual void saveConstr(ostream&);
  public:
    static int addClassCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
  private:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
};

extern const GString gfGftID;

class GFTclTop: public GFTop, public GTclClass {
    // CONCEPT: Furniture graphics kernel
    //
    // ABSTRACT PURPOSE: Base class for Tcl entensible 
    // top classes.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  protected:
    GFTclTop(const char *, GFPrimitive*, GTclClassDesc*, char*[]);

    virtual bool isA(const GString& pString) const {
	return(GFTop::isA(pString) || GTclClass::isA(pString));
    }

    virtual const char* getDescription(void) const { return(GTclClass::getDescription()); }
    virtual const char* getClass(void) const { return(GTclClass::getClass()); }

    virtual void saveConstr(ostream&);
    virtual void saveChildren(ostream& o) {
	GFPrimitive::saveChildren(o);
    }
  public:
    static int addClassCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);
  private:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
};

// initialize Tcl and return true if successful:
bool gfInitTcl(Tcl_Interp*);

#endif

#endif
