#ifndef __GT_BOX_H__
#define __GT_BOX_H__

class GTABox {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: a class for axis aligned boxes
    //
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 

 public:
    GTABox(void) {}
    GTABox(const GPoint3& pMin, const GPoint3& pMax)
      : mMin(pMin), mMax(pMax) {}
   
    GTABox(const GTABox &pBox) {
	mMin = pBox.mMin;
	mMax = pBox.mMax;
    }
   
    // test for overlaping
    bool isOverlap(const GPoint3& point) const;
    bool isOverlap(const GTABox&  box) const;
   
    // compute the surface area 
    GFloat getSurfaceArea(void) const;
   
    // intersect w/ a ray. If hit, tmin & tmax are 2 I-distances  
    bool findMinMax(const GRay3& ray, GFloat &tmin, GFloat &tmax);

    // INCLUDE a point
    GTABox& operator +=(const GPoint3& newpt);
    // INCLUDE an aabox
    GTABox& operator +=(const GTABox& bbox);
   
    // UNION
    friend GTABox operator &(const GTABox& pA, const GTABox& pB) {
	if (pA.isOverlap(pB)) 
	  return(GTABox(getMax(pA.mMin, pB.mMin), getMin(pA.mMax, pB.mMax)));
	else	       
	  return(GTABox(GPoint3(0, 0, 0), GPoint3(0, 0, 0)));
    }

    // INTERSECTION
    friend GTABox operator |(const GTABox& pA, const GTABox& pB) {
	return (GTABox(getMin(pA.mMin, pB.mMin), getMax(pA.mMax, pB.mMax)));
    }

    // INTERFACE
    void min(const GVector3& pMin) { mMin = pMin; } 
    GVector3& min(void) { return(mMin); }
    const GVector3& min(void) const { return(mMin); }

    void max(const GVector3& pMax) { mMax = pMax; } 
    GVector3& max(void) { return(mMax); }
    const GVector3& max(void) const { return(mMax); }
 
 protected:
    GPoint3 mMin, mMax;
};

inline GTABox& GTABox::operator+=(const GPoint3& pPt) {
   mMin.setMin(pPt);
   mMax.setMax(pPt);
   return(self);
}

inline GTABox& GTABox::operator+=(const GTABox& pBox) {
   mMin.setMin( pBox.mMin);
   mMax.setMax( pBox.mMax);
   return(self);
}

inline GFloat GTABox::getSurfaceArea() const  {
   GPoint3 tExt = mMax - mMin;
   return (tExt.getX() * (tExt.getY() + tExt.getZ())+ 
	   tExt.getY() * tExt.getZ());
}

inline bool GTABox::isOverlap(const GTABox& y) const {
   return !(mMax.getX() < y.mMin.getX() || mMin.getX() > y.mMax.getX() ||
	    mMax.getY() < y.mMin.getY() || mMin.getY() > y.mMax.getY() ||
	    mMax.getZ() < y.mMin.getZ() || mMin.getZ() > y.mMax.getZ()  );
}

inline bool GTABox::isOverlap(const GPoint3& pt) const {
    return (mMax.getX() < pt.getX() || mMin.getX() > pt.getX() ||
	    mMax.getY() < pt.getY() || mMin.getY() > pt.getY() ||
	    mMax.getZ() < pt.getZ() || mMin.getZ() > pt.getZ()  );
}


#endif
