
#include <stdlib.h>
#include "render/graytrac.h"

#include "gt_defs.h"
#include "gt_abox.h"
#include "gt_bbox.h"

#include "gt_heap.h"
#include "gt_stack.h"



// KayKaijya BVH-Traversing 
class KKNode   {
 public:
    GFloat mT;
    GTObject *mObj;
    KKNode(void)   {}
    KKNode(GTObject *pObj, GFloat pT) {
	mObj = pObj;
	mT = pT;
    }
    friend int operator <(const KKNode& x, const KKNode& y) {
	return (x.mT < y.mT);
    }
    friend int operator >(const KKNode& x, const KKNode& y) {
	return (x.mT > y.mT);
    }
    friend int operator ==(const KKNode& x, const KKNode& y) {
	return (x.mT == y.mT);
    }
};
typedef Heap<KKNode> __KKHeap;



class StkNode {
 public:  
   GTObjectList *mList;
   long mN;
};
typedef Stack<StkNode> ListStk;
#define GET_OBJ(node) (*((node).mList))[(node).mN]



bool GTABox::findMinMax(const GRay3& pRay, GFloat &rTmin, GFloat &rTmax) {
    GFloat   tT;
    GPoint3  tOrg(pRay.getPoint());
    GVector3 tDir(pRay.getVector());
    
    rTmin = pRay.getTmin();
    rTmax = pRay.getTmax();
   
    // X-Axis 
    if (tDir.getX() < -gEpsilon) {   
	tT = (mMin.getX() - tOrg.getX()) / tDir.getX();
	if (tT < rTmin)		     
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
	
	tT = (mMax.getX() - tOrg.getX()) / tDir.getX();
	if (tT >= rTmin) {
	    if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    } else if (tDir.getX() > gEpsilon) {
	tT = (mMax.getX() - tOrg.getX()) / tDir.getX();
	if (tT < rTmin)
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
	tT = (mMin.getX() - tOrg.getX()) / tDir.getX();
	if (tT >= rTmin){
	 if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    } else if (tOrg.getX() < mMin.getX() || tOrg.getX() > mMax.getX())
      return FALSE;
   
    // Y-Axis 
    if (tDir.getY() < -gEpsilon) {
	tT = (mMin.getY() - tOrg.getY()) / tDir.getY();
	if (tT < rTmin)
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
	tT = (mMax.getY() - tOrg.getY()) / tDir.getY();
	if (tT >= rTmin){
	    if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    } else if (tDir.getY() > gEpsilon) {
	tT = (mMax.getY() - tOrg.getY()) / tDir.getY();
	if (tT < rTmin)
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
	tT = (mMin.getY() - tOrg.getY()) / tDir.getY();
	if (tT >= rTmin){
	    if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    } else if (tOrg.getY() < mMin.getY() || tOrg.getY() > mMax.getY())
      return FALSE;
   
    // Z-Axis 
    if (tDir.getZ() < -gEpsilon) {
	tT = (mMin.getZ() - tOrg.getZ()) / tDir.getZ();
	if (tT < rTmin)
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
      
	tT = (mMax.getZ() - tOrg.getZ()) / tDir.getZ();
	if (tT >= rTmin){
	    if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    } else if (tDir.getZ() > gEpsilon) {
      
	tT = (mMax.getZ() - tOrg.getZ()) / tDir.getZ();
	if (tT < rTmin)
	  return FALSE;
	if (tT <= rTmax)
	  rTmax = tT;
      
	tT = (mMin.getZ() - tOrg.getZ()) / tDir.getZ();
	if (tT >= rTmin){
	    if (tT > rTmax)
	      return FALSE;
	    rTmin = tT;
	}
    }else if (tOrg.getZ() < mMin.getZ() || tOrg.getZ() > mMax.getZ())
      return FALSE;
   
 
    return TRUE;
}
 



// 
// Calculate the cost for a bounding volume.
//
static void __computeCostOfNode(GTBBox *rBound, GFloat& rCost) {
    GTObjectList *tList = rBound->getListOfChildren();
    rCost += tList->count() * rBound->getCost();
   
    for (GTObjectItr _Itr(*tList); 
	 _Itr.more(); 
	 _Itr.next())
     {
	 GTObject *tObj =  _Itr.cur();
	 if(tObj->isBound())
	   __computeCostOfNode( (GTBBox *)tObj, rCost);
     }
}

class SortNode { 
 public:
    GTObject *mObj;
    GFloat    mCost;
};
int __compareNodes(const void *p0,const void *p1 )  {
    SortNode *tNode0  = (SortNode *) p0;
    SortNode *tNode1  = (SortNode *) p1;
    
    if ( tNode0->mCost == tNode1->mCost )  
      return(0);
    else 
      return( tNode0->mCost < tNode1->mCost ? -1 : 1 ) ;
}


// 
// Sort a list of GTObjects. The sorting criteria is the intersection
// cost of the objects. 
//
static void __sortList(GTObjectList *rList) {
    long tNum = rList->count();
    SortNode *tNodes = new SortNode[tNum];
    
    for(register int i = 0; i < tNum; i++) {
	GTObject *tObj  = (*rList)[i];
	tNodes[i].mObj  = tObj;
	tNodes[i].mCost = 0.0;
       
	if(tObj->isBound())
	  __computeCostOfNode((GTBBox *)tObj, tNodes[i].mCost);
	else
	  tNodes[i].mCost = tObj->getCost();
    }
    
    qsort( tNodes, tNum, sizeof( SortNode ), __compareNodes) ;
   
    for(i = 0; i < tNum; i++) 
      (*rList)[i] = tNodes[i].mObj;
   
    delete [] tNodes;
}

static void __optimizeNode(GTBBox *rBound) {
    GTObjectList *tList = rBound->getListOfChildren();
    
    __sortList(tList);
   
    for (GTObjectItr _Itr(*tList); 
	 _Itr.more(); 
	 _Itr.next())
      {
	  GTObject *tObj =  _Itr.cur();
	  if(tObj->isBound())
	    __optimizeNode( (GTBBox *)tObj);
      }
}


// 
// Calculate the cost of a BV-hierarchy.  
//
static GFloat __computeCostOfHierarchy(const GTObjectList& pHier) {
    
    // If root is empty or its head isnt a bound, then exit! 
    if (pHier.count() == 0)
      return 0;
   
    GTObject *tRoot = pHier[0];
    if (! tRoot->isBound())
      return 0;
   
    GFloat tRet = 0;
    __computeCostOfNode((GTBBox *)tRoot, tRet);
   
    tRet = tRet / ((GTBBox *)tRoot)->getCost();
   
   return tRet;
}



//
// Find the best methode to connect a new object to a local position
// in the BVH.
//
static int __findLocalMethod(GTObject*       pOldParent,
			     const GTABox&   pNewChildBound,
			     GFloat&         rCost,
			     const ListStk&  pStk) 
{

    int    tNum = pOldParent->getNumOfChildren();
    GTABox tOldParentBound(pOldParent->getABox());
   
   
    //////////////////////////////////////////////
    // First thing: calculate the local cost!
    //////////////////////////////////////////////
   
    // Calculate the cost for the new parent bound 
    GFloat tCost_NewParentBound = 
      (tOldParentBound | pNewChildBound).getSurfaceArea();
    
    /* 
     * Methode 0:
     * ------------------------------------------------
     * Add the new object direct to the parent bbox,
     * wich may increase in size.
     * 
     *           New_Parent_Bound
     *                 |
     * +---------------+-------+----------------------+
     * |                       |                      |
     * Old_Child[0] ... Old_Child[tNum-1]            New_Child
     * 
     */
    GFloat tCost_M0 = tCost_NewParentBound * (tNum + 1);
    
    /* 
     * Methode 1: 
     * --------------------------------------------------
     * Create a new bound, wich contains the old bound 
     * and the new object.
     * 
     *                      New_Parent_Bound
     *                              |
     *             +----------------+-----------------+
     *             |                                  |
     *      Old_Parent_Bound                       New_Child
     *             |
     * +-----------+-----------+
     * |                       |
     * Old_Child[0] ... Old_Child[tNum-1]
     * 
     */
    GFloat tCost_M1 = 
      2 * tCost_NewParentBound + tNum * tOldParentBound.getSurfaceArea();
   
    int tBestMethode;
    if (tCost_M0 < tCost_M1)  {
	rCost = tCost_M0;
	tBestMethode = 0;
    }
    else  {
	rCost = tCost_M1;
	tBestMethode = 1;
    }
   
    ////////////////////////////////////////////////
    // OK, time to find the inheritance cost... 
    ////////////////////////////////////////////////
   
    // Dont destroy my STACK Plzzzzz! 
    ListStk tStkUp(pStk);
   
    while (!tStkUp.empty())  {
	StkNode tStkNode = tStkUp.pop();
      
	GTObject *tObj = GET_OBJ(tStkNode);
	tNum = tObj->getNumOfChildren();
	rCost += 
	  tNum * (tObj->getABox() | pNewChildBound).getSurfaceArea();
    }
    return tBestMethode;
}
 
//
// Find in the list for the best object to combine w/ the new object  
//
static  long __findBestLocalObj(GTObjectList*  pTestList, 
				const GTABox&  pNewBound)
{
    GFloat  tMinCost = MAXFLT;
    long tRet = 0, tIndex = 0;
   
    for(GTObjectItr _Itr(*pTestList);
	_Itr.more(); _Itr.next(), tIndex++)
      {
	  GFloat tCost = 
	    (_Itr.cur()->getABox()| pNewBound).getSurfaceArea();
      
	  if (tCost < tMinCost)  {
	      tMinCost = tCost;
	      tRet = tIndex;
	  }
      }
    
    return tRet;
}


static void __connectObject(GTObjectList* rList, 
			    GTObject*     pNewObj) 
{
    StkNode        tStkNode;
    static ListStk sSTK;
   
    GTABox        tNewChildBound(pNewObj->getABox());
    GTObjectList* tCurList = rList;
    GTObject*     tObj;
   
    ///////////////////////////////////////////////////////////// 
    // Make a DOWN-path by searching in the hierarchy for 
    // smallest bounds when adding the new object to them.
    /////////////////////////////////////////////////////////////
    for(;;)  {
	tStkNode.mN    = __findBestLocalObj(tCurList, tNewChildBound);
	tStkNode.mList = tCurList;
	sSTK.push(tStkNode);
      
	tObj = GET_OBJ(tStkNode);
	if (tObj->isBound())
	  tCurList = ((GTBBox *)tObj)->getListOfChildren();
	else
	  break;
    } 
   
   
    //////////////////////////////////////////////////////
    // Traverse the path, UP to the root. For each node 
    // in the path, calculate the new intersection cost
    // when the new object is connected to it. The best
    // node and methode will be saved.
    //////////////////////////////////////////////////////
    
   
    // save the stack 
    ListStk tStkUp(sSTK);

    int tBestMethod, tMethod;
    GFloat tMinCost, tCost;
   
    StkNode tMinNode = sSTK.pop();
   
    tObj = GET_OBJ(tMinNode);
    tBestMethod = __findLocalMethod(tObj, 
				    tNewChildBound, 
				    tMinCost, 
				    sSTK);
    while (! sSTK.empty())  {
	
	tStkNode = sSTK.pop();
      
	tObj = GET_OBJ(tStkNode);
	tMethod = __findLocalMethod(tObj, 
				    tNewChildBound, 
				    tCost, 
				    sSTK);
      
	if (tCost < tMinCost)  {
	    tMinNode = tStkNode;
	    tMinCost = tCost;
	    tBestMethod = tMethod;
	 
	    // reconfigure the old stack pointer for expanding the
	    // bounds of the parents
	    while(tStkUp.top().mList != tStkNode.mList)
	      tStkUp.del_top();
	}
    }
      
    tObj = GET_OBJ(tMinNode);

    ///// METHODE 0
    if (!tBestMethod && tObj->isBound())  {
	((GTBBox *)tObj)->add(pNewObj);
    }
    ///// METHODE 1
    else  {
	GTBBox *tNewBound = new GTBBox();
	tNewBound->add(tObj);
	tNewBound->add(pNewObj);
	(*(tMinNode.mList))[tMinNode.mN] =  tNewBound;
    }
   
   
    //////////////////////////////////////////////////////
    // expand the parents bounds to fit the new object 
    //////////////////////////////////////////////////////
    while (! tStkUp.empty())  {
	tStkNode = tStkUp.pop();
	tObj = GET_OBJ(tStkNode);
	if (tObj->isBound())  
	  *((GTBBox *)tObj) += tNewChildBound;
    }
}



static void __buildHierarchy(GTObjectList*       rOptiList, 
			     const GTObjectList& pInputList) 
{
    GTObjectItr _Itr(pInputList);
   
    // remember, item 0 is the root box! 
    rOptiList->append(_Itr.cur());
   
    for(_Itr.next(); _Itr.more(); _Itr.next()) 
      __connectObject(rOptiList, _Itr.cur());
}


//
// Shuffle a list of objects to ensure a better tree
// After G&S.
//
static void __shuffleList(GTObjectList& rList) {
    long tNum = rList.count();
    for(register int i = 0; i < tNum; i++) {
	int tR = rand() % tNum;
	rList.swap(i, tR);
    }
}

void GTBBox::buildHierarchy(void) {
   
    // We dont need to make the BVH when this bound containts
    // only one object!
    if (mListChildren.count() <= 1)
      return;
   
    // Make a copy for the hierarchy 
    GTObjectList tTestList; 
      {
	  for (GTObjectItr _Itr(mListChildren); 
	       _Itr.more(); _Itr.next())
	    tTestList.append(_Itr.cur());
      }
   
    GFloat tBestCost = MAXFLT;
    int tNumTrees = (int)ceil(log(mListChildren.count())/log(2));
    int i;
    for (i = 0; i < tNumTrees; i++)  {
      
	// randomize the list...
	__shuffleList(tTestList);
      
	// build the BVH...
	GTObjectList tOptiList;
	__buildHierarchy(&tOptiList, tTestList);
      
	// Is the cost for this BVH better than the actual 
	// best cost?
	GFloat tTestCost = __computeCostOfHierarchy(tOptiList);
	if (tTestCost < tBestCost)  {
	    tBestCost = tTestCost;
	 
	    // update the BVH for this node 
	    mListChildren.reset();
	      {	 
		  for (GTObjectItr _Itr(tOptiList);
		       _Itr.more(); _Itr.next()) 
		    mListChildren.append(_Itr.cur());
	      }
	}
      
    }
   
    // Sort the list of children 
    __optimizeNode((GTBBox *)(mListChildren[0]));
}



bool GTBBox::intersectNearest(const GRay3&         pRay, 
			      GInterSectionList&  rInter)  
{
    // KayKajia heap
    static  __KKHeap  sKK_HEAP;

#ifdef GTD_LAST_NEAREST
    // last nearest caching
    static  GTObject*  sLastNearest = 0;  
#endif
   
    GTObject *tObj;
    GFloat  tTmin, tTmax, tMinT;
    bool tHIT = FALSE;

#ifdef GTD_LAST_NEAREST
    if(sLastNearest && sLastNearest->intersect(pRay, rInter)) {
	tMinT = rInter.getFirst()->getT();
	tHIT  = TRUE;
    } else {
#endif
	
	// opps, nothing to intersect with..
	if (!mListChildren.count())   
	  goto L_EXIT;
      
	// get the root-node
	tObj = mListChildren[0];
      
	// bound
	if (tObj->isBound()) {
	    if(((GTBBox *)tObj)->findMinMax(pRay, tTmin, tTmax)) {
		sKK_HEAP.insert(KKNode(tObj, tTmin));
		//pRay.minmax(tTmin, tTmax);
		tMinT = tTmax;
	    } else
	      goto L_EXIT;
	}
	// primitive
	else if(tObj->intersect(pRay, rInter)){
	    tMinT = rInter.getFirst().getT();

#ifdef GTD_LAST_NEAREST
	    sLastNearest = tObj;
#endif

	    tHIT = TRUE;
	}

#ifdef GTD_LAST_NEAREST
    }
#endif   
   
    while (sKK_HEAP.count())  {
      
	KKNode tNode = sKK_HEAP.remove();
      
	// There is no need for intersecting w/ the rest of bounds
	// when the nearest object was found.
	if ((tNode.mT - tMinT) > gEpsilon )  
	  break;             

	GTObjectList *tList = 
	  ((GTBBox *)tNode.mObj)->getListOfChildren();
      
	for(GTObjectItr _Itr(*tList);
	    _Itr.more();
	    _Itr.next()) 
	  {
	      tObj = _Itr.cur();
	   
	      // If the object is a bound and it intersects
	      // the ray, insert it to the KK-Heap w/ _tmin
	      // as the key.
	      
	      // bound
	      if (tObj->isBound()) {
		  if(((GTBBox*)tObj)->findMinMax(pRay, tTmin, tTmax)) {
		      sKK_HEAP.insert(KKNode(tObj, tTmin));
		  }
	      }
	      // primitive
	      else  if(tObj->intersect(pRay, rInter)) {
		  tMinT = rInter.getFirst().getT();
#ifdef GTD_LAST_NEAREST
		  sLastNearest = tObj;
#endif
		  tHIT = TRUE;
	      }

	  }
    }
   
    // L_FREE_HEAP:
    sKK_HEAP.reset();
   
    L_EXIT:
    return tHIT;
}

bool GTBBox::intersect(const GRay3&        pRay, 
		       GInterSectionList&  rInter)
{
    int tHIT = FALSE;
 
    for(GTObjectItr _Itr(mListChildren);
	_Itr.more(); _Itr.next()) 
      {
	  GTObject *tPrim = _Itr.cur();
	
	  if(tPrim->intersect(pRay, rInter))
	    tHIT = TRUE;
     }
    
    return tHIT;
}

// Dtor
GTBBox::~GTBBox(void) 
{
    for(GTObjectItr _Itr(mListChildren);
	_Itr.more(); _Itr.next()) {
	GTObject *tObj = _Itr.cur();
	if(tObj->isBound())
	  delete tObj;
    }
}

 

