#ifndef __GT_BBOX_H__
#define __GT_BBOX_H__

#include "gt_abox.h"
#include "gt_obj.h"

class GTBBox : public GTABox, public GTObject {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Implements a class for bounding boxes
    //
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 

 public:
   
    GTBBox(void)
      : GTObject(GTObject::O_BOUND),
      GTABox(GPoint3(MAXFLT), GPoint3(-MAXFLT))  {}
   
    GTBBox(const GPoint3& pMin, const GPoint3& pMax)
      : GTObject(GTObject::O_BOUND),
      GTABox(pMin, pMax) {}
   
    GTBBox(GTABox& pBox)
      : GTObject(GTObject::O_BOUND),
      GTABox(pBox)  {}
   
    virtual ~GTBBox(void);
   
    bool     intersect(const GRay3&, GInterSectionList&); 
    GVector3 getNormal(const GPoint3&) {
	return (GVector3(0, 0, 0));
    }

    GFloat   getCost(void) const;
    void     add(GTObject *);
   
    bool     isOverlap(const GPoint3& point) const;
    bool     isOverlap(const GTABox&  box) const;
    GTABox   getABox(void) const;  

    void     buildHierarchy(void);
    bool     intersectNearest(const GRay3 &, GInterSectionList &);

   
    GTObjectList  *getListOfChildren(void);
    int            getNumOfChildren(void) const; 


 protected:
    GTObjectList mListChildren;
   
};

inline GFloat GTBBox::getCost(void) const {
   return (GTABox::getSurfaceArea());
}

inline GVector3 getNormal(const GPoint3&) {
    STAT( cerr << "Error: GTBox::getNormal was called!\n"; );
    return GVector3(0, 0, 1);
}

inline void GTBBox::add(GTObject *pObj) {
    // expand the actual bounding box
    (self)+= pObj->getABox();
   
    // Add the new object to the list of children
    mListChildren.append(pObj);
}

inline GTObjectList* GTBBox::getListOfChildren(void) {
    return (&mListChildren);
}

inline int GTBBox::getNumOfChildren(void) const { 
   return (mListChildren.count());
}

   
inline bool GTBBox::isOverlap(const GPoint3& pPoint) const {
    return (GTABox::isOverlap(pPoint));
}

inline bool GTBBox::isOverlap(const GTABox&  pBox) const {
    return (GTABox::isOverlap(pBox));
}

inline GTABox GTBBox::getABox(void) const {
    return (self);
}  

#endif
