#ifndef __GT_HEAP_H__
#define __GT_HEAP_H__


template<class T>
class Heap   {  
 public:  
    Heap(int pSize=2);  
    ~Heap(void);  
    
    void     insert(const T& x);  
    T        remove(void);  
    void     reset(void);  
    int      count(void) const;  
    
 private:  
    T*       mItems;  
    int      mNumber;  
    int      mSize;  
    
    void     upHeap(int i);  
    void     downHeap(int i);  
};  
 
template<class T> 
inline Heap<T>::Heap(int pSize)  {  
    mItems  = new T[pSize];  
    mNumber = 0;  
    mSize   = pSize;  
}  

template<class T>
inline Heap<T>::~Heap()  {  
    delete[] mItems;  
}  
 
template<class T> 
inline void Heap<T>::insert(const T& pItem)  {  
    if (++mNumber >= mSize)   {  
	T *temp = new T[mSize <<= 1];  
	for (register int i = 0; i < mNumber-1; i++)  
	  temp[i] = mItems[i];  
	delete[] mItems;  
	mItems = temp;  
    }  
    mItems[mNumber-1] = pItem;  
    upHeap(mNumber-1);  
}  
 
template<class T>
inline T Heap<T>::remove(void)  {  
    T ret = mItems[0];  
    if (--mNumber > 0)   {  
	mItems[0] = mItems[mNumber];  
	downHeap(0);  
    }  
    return(ret);  
}  

template<class T>
inline void  Heap<T>::reset(void)  {  
    mNumber = 0;  
}  
 
template<class T> 
inline int Heap<T>::count(void) const  {  
    return(mNumber);  
}  
 
template<class T>
inline void  Heap<T>::upHeap(int i)  {  
    int p;  
    T temp = mItems[i];  
    
    while (i)   {  
	p = (i-1) >> 1;  
	if ( mItems[p] < temp )  
	  break;  
	mItems[i] = mItems[p];  
	i = p;  
    }  
    mItems[i] = temp;  
}  
 
template<class T>
inline void Heap<T>::downHeap(int i)  {  
    int c;  
    T temp = mItems[i];  
    
    while((c = ((i<<1)+1)) < mNumber)   {  
	if (c+1 < mNumber)  
	  c += (mItems[c+1] < mItems[c]) ;  
	if (temp < mItems[c] )  
	  break;  
	mItems[i] = mItems[c];  
	i = c;  
    }  
    mItems[i] = temp;  
}


#endif
