
#include "gt_defs.h"
#include "gt_shade.h"
#include "gt_cam.h"

#include "gt_light.h"

GDLList<GTLight>* gGTLights = 0;

/*
 * @FUNCT@: @RT_PointLight::shade@
 * 
 * @DESCR@: shade a point by "first order"-illumination
 * 
 * @NOTES@: Shadow-cache (Haines & Co.:-)
 * 
 */
GColorRGB GTPointLight::shade(GTShadingParams &rSP)
{
    if (!isOn()) 
      return(GColorRGB( 0, 0, 0 ));
   
    GColorRGB tColor(0, 0, 0);
	
    // calculate the shadow rays direction 
    GVector3 tDir(getPosition() - rSP.mP);   
    // calculate the distance from rays origin to the given point 
    GFloat   tDist = tDir.abs(); 
   
    // normalize the rays direction
    tDir /= tDist;
   
    // We are going to fire a shadow ray! 
    GRay3    tShadowRay(rSP.mP, tDir, GRay3::R_SHADOW);
    // update tmin and tmax. Significant interval is [0, _dis] 
    tShadowRay.minmax(gEpsilon, tDist);
   
    // First, check the last opaque object for intersection 
    if (mLastOpaque)  {
	if (mLastOpaque->isHit(tShadowRay))  {
	    return tColor;
	}else  {
	    mLastOpaque = 0;
	}
    }
   
    // shadow cache missed! Test for shadow w/ all objects in the scene
    // If a opaque object was found, then update the cache. 
    GColorRGB tAtten = rSP.mRayTracer->traceShadowRay(tShadowRay, 
						      mLastOpaque);
   
    // a new opaque object intersects the ray! 
    if (mLastOpaque)  
      return tColor;
    
    
    // update the light vector 
    rSP.mL = tDir;
    
    // multiple the light-color w/ all attenuation factors. We are done! 
    tColor += shadeDiffuse(rSP);
    tColor += shadeSpecular(rSP);
    return tColor * getColor() * tAtten * findDistAtten(tDist);
}
 

GColorRGB GTPointLight::shadeDiffuse(const GTShadingParams &pSP)
{
    GFloat tDiffAtten = pSP.mN.dot(pSP.mL);
    return ((tDiffAtten > 0) ? pSP.mSurface->getDiffuse() * tDiffAtten 
	    : GColorRGB(0, 0, 0));
}

GColorRGB GTPointLight::shadeSpecular(const GTShadingParams &pSP)
{
    GFloat tSpecCosine = pSP.mR.dot(pSP.mL);
    return (tSpecCosine > 0) ? 
      pSP.mSurface->getSpecular() * 
      pow(tSpecCosine, pSP.mSurface->getShininess())
      : GColorRGB(0, 0, 0);
}

GColorRGB GTAmbientLight::shade(GTShadingParams &pSP) { 
    if (!isOn()) 
      return (GColorRGB( 0, 0, 0 ));
    else
      return (getColor() * pSP.mSurface->getAmbient());
}
