#ifndef __GT_LIGHT_H__
#define __GT_LIGHT_H__

#include "render/glight.h"
#include "gt_shade.h"






class GTLight: public GLight {
 public:
   
    GTLight(void)
      : GLight() { }
   
    virtual GColorRGB shade           (GTShadingParams& rSP)=0;
    
    virtual GColorRGB shadeDiffuse    (const GTShadingParams &pSP) {
	return (GColorRGB(0, 0, 0));
    }
    virtual GColorRGB shadeSpecular   (const GTShadingParams &pSP) {
	return (GColorRGB(0, 0, 0));
    }
};



class GTPointLight: public GTLight {
 public:
   
    GTPointLight(void)
      : GTLight(), mC1(1), mC2(0), mC3(0), mLastOpaque(0) { }
   
    GColorRGB shade           (GTShadingParams& rSP);
    GColorRGB shadeDiffuse    (const GTShadingParams &pSP);
    GColorRGB shadeSpecular   (const GTShadingParams &pSP);
    
    GFloat findDistAtten(GFloat pT) {
	GFloat tRet = 1 / (mC1 + pT * (mC2 + mC3 * pT));
	return ((tRet < 1) ? tRet : 1);
    }
 private:
    // lights coefficients for attenuation 
    GFloat mC1, mC2, mC3;              
    // reserved for shadow caching 
    GRayTraceable *mLastOpaque;    
};

class GTAmbientLight: public GTLight {
  public:
    GTAmbientLight(void) 
      :GTLight() {}
    GColorRGB  shade(GTShadingParams &rSP);
};


extern GDLList<GTLight>* gGTLights;

#endif
