
#ifndef __RTE_SHADE_H__
#define __RTE_SHADE_H__


/*
 * @CLASS@: GTShadingParams
 * 
 * @DESCR@: This class containts all parameters for shading calculation
 *  P -> point to shade
 *  N -> normal vector at this point
 *  L -> incident light-vector
 *  R -> specular reflection's vector
 *  V -> view-vector (ray's direction)
 * 
 *  pSurface -> ptr to the surface of P
 *  pCamera  -> ptr to the camera, needed for rays tracing
 * 
 * @NOTES@: For shading-functions, only ONE parameter from this type must be
 * passed anstead a lot of special shading-parameters. The less pushing, the
 * more efficient!
 * 
 * 
 */
class GSurface;
class GTRayTracer;

class GTShadingParams {
 public:
    GVector3     mP, mN, mL, mR, mV; 
    bool         mEnter;
    GSurface*    mSurface;
    GTRayTracer* mRayTracer;
    
    GTShadingParams(const GVector3& pP, const GVector3& pN, const GVector3& pL,
		   const GVector3& pR, const GVector3& pV, bool pEnter,
		   GSurface* pSurf, GTRayTracer* pRT)
      :mP(pP), mN(pN), mL(pL), mR(pR), mV(pV),
      mEnter(pEnter),mSurface(pSurf), mRayTracer(pRT) {}
};

#endif
