#ifndef __GT_STACK_H__
#define __GT_STACK_H__

#define DEFAULT_STACK_SIZE 1024

template<class T> 
class Stack  { 
 public: 
      
    Stack(int sz = DEFAULT_STACK_SIZE); 
    Stack(Stack<T>&); 
    ~Stack(void); 
      
    void                operator =(Stack<T>&); 
    void                push( T& item); 
    T                   pop(void); 
    T&                  top(void); 
    void                del_top(void); 
      
    int                 length(void) const; 
    int                 empty(void) const; 
    int                 full(void) const; 
    void                reset(void); 
    void                resize(int sz); 

 protected: 
    int                 mSize; 
    int                 mPtr; 
    T*                  mItems; 
}; 

template<class T>
inline Stack<T>::Stack(int pSize) { 
    mItems = new T [mSize = pSize]; 
    mPtr = 0; 
} 

template<class T>
inline Stack<T>::~Stack(void) { 
    delete [] mItems; 
} 

template<class T>
inline void Stack<T>::reset(void) {
    mPtr = 0; 
} 

template<class T>
inline int Stack<T>::empty(void) const { 
    return(mPtr == 0); 
} 

template<class T>
inline int Stack<T>::full(void) const { 
    return(mPtr == mSize); 
} 

template<class T>
inline int Stack<T>::length(void) const { 
    return(mPtr); 
} 

template<class T>
inline void Stack<T>::push(T& pItem) {
    CHECK(full(), "Stack::push() ->FULL"); 
    mItems[mPtr++] = pItem; 
} 

template<class T>
inline T Stack<T>::pop(void) { 
    CHECK(empty(),"Stack::pop() ->EMPTY"); 
    return(mItems[--mPtr]); 
} 

template<class T>
inline void Stack<T>::del_top(void) { 
    CHECK(empty(), "Stack::del_top() ->EMPTY"); 
    --mPtr; 
} 

template<class T>
inline T& Stack<T>::top(void) { 
    CHECK(empty(), "Stack::top() ->EMPTY"); 
    return(mItems[mPtr-1]); 
}

template<class T>
inline Stack<T>::Stack(Stack<T>& b) 
:mSize(b.mSize), mPtr(b.mPtr), mItems(new T [b.mSize]) { 
    for (int i = 0; i < mPtr; ++i) 
      mItems[i] = b.mItems[i]; 
} 

template<class T>
void Stack<T>::operator = (Stack<T>& b) { 
    if (&b == this) return; 
    if (mSize < b.mPtr) { 
	delete [] mItems; 
	mItems = new T [b.mSize]; 
	mSize = b.mSize; 
    } 
    mPtr = b.mPtr; 
    for (int i = 0; i < mPtr; ++i) 
      mItems[i] = b.mItems[i]; 
} 

template<class T>
void Stack<T>::resize(int newsz) { 
    CHECK((newsz < mPtr), "Stack::resize() size too small"); 
    T* news = new T [newsz]; 
    for (int i = 0; i < mPtr; ++i) 
      news[i] = mItems[i]; 
    delete [] mItems; 
    mItems = news; 
    mSize = newsz; 
} 

#endif
