#ifndef G_LIGHT_H
#define G_LIGHT_H

#include "util/gvector.h"
#include "util/gcolor.h"

class GLight {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Base class of all
    // light sources. Rendering independent.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  protected:
    GLight(void): mColor(gWhite), mPosition(gOrigin3), mState(true) {}
  public:
    void setColor(const GColorRGB& pColor) {
	mColor = pColor;
    }
    const GColorRGB& getColor(void) const {
	return(mColor);
    }

    void setPosition(const GPoint3& pPosition) {
	mPosition = pPosition;
    }
    const GPoint3& getPosition(void) const {
	return(mPosition);
    }

    void on(void) {
	mState = true;
    }
    void off(void) {
	mState = false;
    }
    bool isOn(void) const {
	return(mState);
    }
  private:
    GColorRGB mColor;
    GPoint3 mPosition;
    bool mState;
};

class GPointLight: public GLight {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Specification of
    // a point light source.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  protected:
    GPointLight(void): mDirection(GVector3(0.0, 0.0, -1.0)) {}
  public:
    void setDirection(const GVector3& pDirection) {
	mDirection = pDirection;
    }
    const GVector3& getDirection(void) const {
	return(mDirection);
    }
  private:
    GVector3 mDirection;
};

#endif
