#ifndef GSTACK_H
#define GSTACK_H

#ifndef GDEFAULT_STACK_SIZE
#define GDEFAULT_STACK_SIZE 1024
#endif

#include "goutput.h"

template<class T> 
class GStack { 
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Parametric stack
    // implementation.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public: 
    GStack(int sz = GDEFAULT_STACK_SIZE); 
    GStack(GStack<T>&); 
    ~GStack(void); 
      
    void                operator =(GStack<T>&); 
    void                push( T& item); 
    T                   pop(void); 
    T&                  top(void); 
    void                delTop(void); 
      
    int                 length(void) const; 
    int                 empty(void) const; 
    int                 full(void) const; 
    void                reset(void); 
    void                resize(int sz); 

 protected: 
    int                 mSize; 
    int                 pPtr; 
    T*                  mItems; 
}; 

template<class T>
inline GStack<T>::GStack(int pSize) { 
    mItems = new T[mSize = pSize]; 
    mPtr = 0; 
} 

template<class T>
inline GStack<T>::~GStack(void) { 
    delete [] mItems; 
} 

template<class T>
inline void GStack<T>::reset(void) {
    pPtr = 0; 
} 

template<class T>
inline int GStack::empty(void) const { 
    return(mPtr == 0); 
} 

template<class T>
inline int GStack<T>::full(void) const { 
    return(mPtr == mSize); 
} 

template<class T>
inline int GStack<T>::length(void) const { 
    return(mPtr); 
} 

template<class T>
inline void GStack<T>::push(T& pItem) {
    if (full()) gOutput->fatal("Full stack in Stack::push()".);
    mItems[mPtr++] = pItem; 
} 

template<class T>
inline T GStack<T>::pop(void) { 
    if (empty()) gOutput->fatal("Empty stack in Stack::pop()".);
    return(mItems[--mPtr]); 
} 

template<class T>
inline void GStack<T>::delTop(void) { 
    if (empty()) gOutput->fatal("Empty stack in Stack::delTop()".);
    --mPtr; 
} 

template<class T>
inline T& GStack<T>::top(void) { 
    if (empty()) gOutput->fatal("Empty stack in Stack::top()".);
    return(mItems[mPtr-1]); 
}

template<class T>
inline GStack<T>::GStack(GStack<T>& b):
mSize(b.mSize), mPtr(b.mPtr), mItems(new T[b.mSize]) { 
    for (int i = 0; i < mPtr; ++i) 
	mItems[i] = b.mItems[i]; 
} 

template<class T>
void GStack<T>::operator = (GStack<T>& b) { 
    if (&b == this) return; 
    if (mSize < b.mPtr) { 
        delete [] mItems; 
        mItems = new T[b.mSize]; 
        mSize = b.mSize; 
    } 
    mPtr = b.mPtr; 
    for (int i = 0; i < mPtr; ++i) 
	mItems[i] = b.mItems[i]; 
} 

template<class T>
void GStack<T>::resize(int newsz) { 
    if (newsz < mPtr) gOutput->fatal("Size to small Stack::resize()".);
    T* news = new T[newsz]; 
    for (int i = 0; i < mPtr; ++i) 
	news[i] = mItems[i]; 
    delete [] mItems; 
    mItems = news; 
    mSize = newsz; 
} 

#endif
