#ifndef G_STRING_H
#define G_STRING_H

#include "gdata.h"
#include <string.h>

class GString: public GStorable {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: A class encapsulating C strings.
    //
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
    friend ostream& operator<<(ostream&, const GString&);
  public:
    GString(const char*);
    // CTOR: Construct string from C string.
    GString(const GString&);
    // CTOR: Copy constructor.
    GString(int = 10);
    // CTOR: Default constructor with an optional length reservation.

    virtual ~GString(void) { delete mChars; }

    // Assignment operator:
    GString& operator=(const GString&);

    // Equality operator:
    bool operator==(const GString&) const;
    bool operator==(const char*) const;

    // Append a string/character on the end:
    const GString& operator+=(const char*);
    const GString& operator+=(const GString& s);
    const GString& operator+=(char);

    operator char* (void) const { return(mChars); }
    // METHOD: Overloaded type-cast for interfacing with C:

    GString substr(int i1, int i2);
    // METHOD: Compute sub string from index $$i1$$ to $$i2$$.

    void del(int i1, int i2);
    // METHOD: Delete sub string of from index $$i1$$ to $$i2$$.

    int getLength(void) const { return(strlen(mChars)); }
    // METHOD: Return current string length.
  private:
    int mLength;
    // Size of reserved memory.
    char *mChars;
};

#endif
 
