#ifndef G_SURFACE_H
#define G_SURFACE_H

#include "util/gcolor.h"

class GSurface {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: An important surface type.
    // However, there can be others.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
    friend ostream& operator<<(ostream&, const GSurface&);
  public:
    GSurface(void);
    // CTOR: Create a surface with default parameters.
 
    void setAmbient(const GColorRGB& pAmbient) { mAmbient = pAmbient; }
    const GColorRGB& getAmbient(void) const { return mAmbient; }

    void setDiffuse(const GColorRGB& pDiffuse) { mDiffuse = pDiffuse; }
    const GColorRGB& getDiffuse(void) const { return mDiffuse; }

    void setSpecular(const GColorRGB& pSpecular) { mSpecular = pSpecular; }
    const GColorRGB& getSpecular(void) const { return mSpecular; }

    void setShininess(GFloat pShininess) { mShininess = pShininess; }
    const GFloat getShininess(void) const { return mShininess; }

    void setRefraction(GFloat pRefraction) { mRefraction = pRefraction; }
    const GFloat getRefraction(void) const { return mRefraction; }

    void setTransparency(GFloat pTransparency) { mTransparency = pTransparency; }
    const GFloat getTransparency(void) const { return mTransparency; }
  private:
    GColorRGB mAmbient, mDiffuse, mSpecular;
    GFloat mShininess, mRefraction, mTransparency;
};

inline GSurface::GSurface(void) {
    // default surface:
    setAmbient(GColorRGB(1.0, 1.0, 1.0));
    setDiffuse(GColorRGB(1.0, 1.0, 1.0));
    setSpecular(GColorRGB(0.0, 0.0, 0.0));
    setShininess(30);
    setRefraction(1.0);
    setTransparency(0.0);
}
    

#endif
