#ifndef G_TCL_H
#define G_TCL_H

#ifdef GWITH_TCL

#include "util/gtclutil.h"

class GTclInterface {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Base class for 
    // interfacing with Tcl.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  protected:
    GTclInterface(const char*);
    virtual ~GTclInterface(void);
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
  protected:
    // Access to result:
    static void setResult(const char* pResult) { sResult = pResult; }
    static const char* getResult(void) { return(sResult); }
  private:
    // To be called from static object command:
    virtual bool objectCMD(char*[]) = 0;
    static int objectCMD(ClientData, Tcl_Interp*, int, char*[]);
  private:
    static GString sResult;
    GString mName;
};

class GTclClass {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Base class for interpretative classes.
    // Has to be used via inheritance in concrete sub classes.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
    friend void gTclInitClassCommands(Tcl_Interp*);
  protected:
    GTclClass(const char*, GTclClassDesc*);
  public:
    virtual ~GTclClass(void);

    bool isA(const char*) const;
    const char* getName(void) const { return(mName); }
    const char* getKeywords(void) const { return(mDesc->getKeywords()); }
    const char* getClass(void) const { return(mDesc->getClass()); }
    const char* getDescription(void) const { return(mDesc->getDescription()); } 
    
    operator const GTclClassDesc*(void) const { return(mDesc); }
  protected:
    static int classCMD(ClientData, Tcl_Interp*, int, char*[]);
    virtual bool objectCMD(char*[]);

    static int addClassCMD(ClientData, Tcl_Interp*, int, char*[]);
  public:
    static int addMethodCMD(ClientData, Tcl_Interp*, int, char*[]);
    static const GDLList<GTclClassDesc>& getClasses(void){
	return(sClasses);
    }
  private:
    // interface for coupling a class browser:
    static int getClassesCMD(ClientData, Tcl_Interp*, int, char*[]);
    static int deleteClassCMD(ClientData, Tcl_Interp*, int, char*[]);

    static int getMethodsCMD(ClientData, Tcl_Interp*, int, char*[]);
    static int deleteMethodCMD(ClientData, Tcl_Interp*, int, char*[]);

    static int getParameterCMD(ClientData, Tcl_Interp*, int, char*[]);
    static int getDescriptionCMD(ClientData, Tcl_Interp*, int, char*[]);
    static int getBodyCMD(ClientData, Tcl_Interp*, int, char*[]);
    static int getParentClassCMD(ClientData, Tcl_Interp*, int, char*[]);

    static int includeClassCMD(ClientData, Tcl_Interp*, int, char*[]);
  protected:
    static GTclClassDesc* getClassByName(const char*);
  protected:
    void addMembers(const char*);
    // Add entries for members into the members list.
    // The string must contain null, one or more names
    // of members separated by spaces.

    bool callMethod(const GTclClassDesc* DESC, char** ARGS, bool CONSTR = false);
    // Call a Tcl method by evaluating the ARGS.
    // If CONSTR is false it is a standard method, else it is the constructor.
    // Returns true if a method could be performed successfully.

    static GTclMethodDesc* find(Tcl_Interp*, int, char *[]);
    // Find a method description of class argv[1] and method argv[2]
    // Returns 0 if nothing was found.
  protected:  
    GString mName;

    GTclClassDesc* mDesc;
    // a link to the related TclClassClassDescription

    GTclMemberList mMembers;
  protected:
    static GDLList<GTclClassDesc> sClasses;
    // a list containing all class descriptions
};

// initialize Tcl class system:
void gTclInitClassCommands(Tcl_Interp*);

#endif

#endif

