#ifndef GTCL_PARSE_H
#define GTCL_PARSE_H

#include <tcl.h>

#include "util/gvector.h"
#include "util/gmatrix.h"
#include "util/gstring.h"
#include "util/gtempl.h"

// the next free enum value to be used in extended kernels: 
extern const int gTclNextParseEnum;

extern const char* gTclHelp;

extern Tcl_Interp* gTclTmpInterp;
// This static temporary Tcl interpreter may be used for
// splitting of argument list.

extern GString gTclTmpString;

extern Tcl_Interp* gGlobalInterp;

void gTclAppendResult(const char*);
int gTclGlobalEval(const char*);
const char* gTclGetResult(void);

typedef enum { 
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Enumeration of primitive
    // parse types. May be extended in an extended
    // graphics kernel.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: gTclNextParseEnum == gTclSpecial
    gTclEnd = 1, 
    gTclNone, 
    gTclString, 
    gTclBool,
    gTclInt, 
    gTclFloat, 
    gTclVector2,
    gTclVector3,
    gTclMatrix4, 
    gTclSpecial 
} GTclParseType;

typedef struct {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Basic element of parse tables.
    // 
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES: Parse tables are created 
    // statically. Therefore, there is no notion of a
    // destructor here.
    // CONSTANTS: 
    char* mName;
    GTclParseType mType;
    char* mParam;
    bool* mIsSet;
    char* mInfo;
    char* mTclTypes;
} GTclParseEntry;

int gTclFindString(char** ARGS, const char *NAME);
// Return position number of NAME in ARGS or -1.

void gTclClearArgv(char** ARGS, int A, int N);
// Remove N strings from position A in ARGS (including the A position).

void gTclParseTable(char** ARGS, GTclParseEntry*);

// The utility functions return true if successful:

bool gTclGetBool(const char**, bool&, int&);
bool gTclGetInt(const char**, int&, int&);
bool gTclGetFloat(const char**, GFloat&, int&);
bool gTclGetString(const char**, const char*&, int&);
bool gTclGetVector2(const char**, GVector2&, int&);
bool gTclGetVector3(const char**, GVector3&, int&);
bool gTclGetMatrix4(const char**, GMatrix4&, int&);

bool gTclString2Bool(const char*, bool&);
bool gTclString2Int(const char*, int&);
bool gTclString2Float(const char*, GFloat&);
bool gTclString2Vector2(const char*, GVector2&);
bool gTclString2Vector3(const char*, GVector3&);
bool gTclString2Matrix4(const char*, GMatrix4&);
bool gTclString2Vector2List(const char*, GDLList<GPoint2>&);

// The return value of the following functions has to be
// copied immediately:
const char* gTclBool2String(bool);
const char* gTclInt2String(int);
const char* gTclFloat2String(GFloat);
const char* gTclVector32String(const GVector3&);

#endif
