#ifndef G_TCL_UTIL_H
#define G_TCL_UTIL_H

#include "util/gtempl.h"
#include "util/gtclpars.h"

#define TCL_HELP 4711

class GTclMethodDesc {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Encapsulation of a method
    // of a class for the interpretative class system.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES: Due to the public data
    // members: this is more a struct in C sense than
    // a real class.
    // CONSTANTS: 
  public:
    GTclMethodDesc(void): mIsMethod(true), mMethod(""), mParams(""), mPTypes(""),
    mDesc(""), mBody(""), mClass("") {}

    const GString& getInformation(void) const;
  public:
    bool mIsMethod;
    // false if the method is a constructor
    GString mMethod;
    GString mParams;
    GString mPTypes;
    GString mDesc;
    GString mBody;
    GString mClass;
};

class GTclMember {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Encapsulation of a data member 
    // of a class for the interpretative class system.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES: 
    //
    // CONSTANTS: 
  public:
    GTclMember(const char* pName, const char* pValue): mName(pName), mValue(pValue) {}

    // get name:
    const char* getName(void) const { return(mName); }

    // set/get value:
    void setValue(const char* pValue) { mValue = pValue; }
    const char* getValue(void) const { return(mValue); }
  private:
    GString mName, mValue;
};

class GTclMemberList {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Management of data members
    // of a class for the interpretative class system.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES:
    //
    // CONSTANTS: 
  public:
    GTclMemberList(void): mMembers(true) {}
    ~GTclMemberList(void) {
	// !TODO! Remove the variables from the global interpreter.
    }

    void set(const char* MEMBER, const char* VALUE);
    // Set a new VALUE for the specified MEMBER.

    void add(const char* MEMBER);
    // Add a new member to the list and initialize it with {}.
    // If it already exists - do nothing.

    void setMembers(const char*);
    // Write members into the global interpreter context.

    void getMembers(const char*) const;
    // Get members from the global interpreter context.

    void globalize(const char*, GString&);
    // Declare it as globals in a local procedure context.
  private:
    GDLList<GTclMember> mMembers; 
};

class GTclClassDesc {
    // CONCEPT: Generic graphics kernel
    //
    // ABSTRACT PURPOSE: Definition of a Tcl class
    // consisting of a constructor and an optional 
    // set of methods. Methods can be added at objects
    // life time.
    // INVARIANT:
    //
    // IMPLEMENTATION ISSUES: 
    // 
    // CONSTANTS: 
  public:
    // Create a class description.
    GTclClassDesc(const char* pFather, const char* pMembers): mFather(pFather), mMembers(pMembers) {}

    // Set/get keywords for the class:
    void setKeywords(const char* pKeywords) {
	mKeywords = pKeywords;
    }
    const char* getKeywords(void) const { return(mKeywords); }

    // Set/get class description:
    void setDescription(const char* pDesc) {
	mDesc = pDesc;
    }
    const char* getDescription(void) const { return(mDesc); }

    // Inquire class:
    const char* getClass(void) const { return(mClass); }

    // Inquire parent class:
    const char* getParentClass(void) const { return(mFather); }

    // Put the related information to the Tcl result:
    void inquireInformation(void) const;

  public:
    GTclMethodDesc mConstr;
    // The constructor description of the class. The class name is 
    // equal to the constructor name.

    GDLList<GTclMethodDesc> mMethods;
    // The remaining method descriptions.

    GString mConstrCmd;

    GString mKeywords; 

    GString mDesc; 

    GString mFather;
    // Name of the father class.

    GString mMembers;
    // List of members (initially empty).

    GString mClass;
};

#endif

