//////////////////////////////////////////////////////////////////////////////
//  This file belongs to the GENERIC 3D kernel. Copying, distribution and     
//  legal info is in the file COPYRIGHT which should be distributed with this 
//  file. If COPYRGHT is not available or for more info please contact: 
//		ekki@prakinf.tu-ilmenau.de
/////////// (C) '95 Ekkehard 'Ekki' Beier - All rights reserved. /////////////
#include "render/graycam.h"

const int gMaxRecursion = 10;
const GFloat gMinWeight = 0.01;

GVector3 gSpecularDirection(const GVector3& i, const GVector3& n) {
    GFloat f = -2.0 * i.dot(n);
    GVector3 r = n.scale(f) + i;
    return(r);
}

int gTransmissionDirection(const GSurface* m1, const GSurface* m2, 
			   const GVector3& i, const GVector3& n, GVector3& t) {
    GFloat n1 = m1 ? m1->getRefraction() : 1;
    GFloat n2 = m2 ? m2->getRefraction() : 1;
    
    GFloat eta = n1/n2; 
    // relative index of refraction
    GFloat cs1 = - i.dot( n ); 
    // cos O1
    GFloat cs2_2 = 1 - eta * eta * (1 - cs1 * cs1); 
    // cos2 O2
    
    if (cs2_2 < 0) return(0); 
    // total internal refraction

    t = i * eta + n * (eta * cs1 - sqrt(cs2_2));

    return(1);
}

void gGetSurfaces(const GSurface& pSurface, int pTraceCnt, bool pEnter, const GSurface*& pEnterS, const GSurface*& pExitS) {
    static const GSurface* sSurface; 
	    
    if (pEnter) {
	pExitS = sSurface;
	pEnterS = &pSurface;
	sSurface = pEnterS;
    }
    else {
	pEnterS = sSurface; 
	pExitS = &pSurface;
	sSurface = pExitS;
    }
    
    // coming from air:
    if (!pTraceCnt && pEnter) pExitS = 0; 
    
    // going into air:
    if ((pTraceCnt == 1) && !pEnter) pEnterS = 0;
}

