//////////////////////////////////////////////////////////////////////////////
//  This file belongs to the GENERIC 3D kernel. Copying, distribution and     
//  legal info is in the file COPYRIGHT which should be distributed with this 
//  file. If COPYRGHT is not available or for more info please contact: 
//		ekki@prakinf.tu-ilmenau.de
/////////// (C) '95 Ekkehard 'Ekki' Beier - All rights reserved. /////////////
#include "util/gstring.h"

ostream& operator<<(ostream& pStream, const GString& pString) {
    pStream << pString.mChars;
    return(pStream);
}

const GString& GString::operator+=(const GString& pString) {
    return(*this += pString.mChars);
}

const GString& GString::operator+=(char c) {	
    int tLength = strlen(mChars) + 2;
    if (tLength > mLength) {
	mLength = tLength + tLength % 16;
	char* tChars = mChars;
	mChars = new char[mLength];
	strcpy(mChars, tChars);
	delete tChars;
    }
    mChars[tLength - 2] = c;
    mChars[tLength - 1] = 0;
    assert(mLength > getLength());
    return(*this);
}

const GString& GString::operator+=(const char* pChars) {
    int tLength = strlen(mChars) + strlen(pChars) + 1;
    if (tLength > mLength) {
	mLength = tLength + tLength % 16;
	char* tChars = mChars;
	mChars = new char[mLength];
	strcpy(mChars, tChars);
	delete tChars;
    }
    strcat(mChars, pChars);
    assert(mLength > getLength());
    return(*this);
}

GString::GString(int pLength): mLength(pLength) {
    // Correct invalid length value:
    if (mLength < 1) mLength = 10;
    mChars = new char[mLength];
    *mChars = 0;
}

GString::GString(const char *pChars) {
    assert(pChars);
    mLength = strlen(pChars) + 1;
    mChars = new char[mLength];
    strcpy(mChars, pChars);
} 

GString::GString(const GString& pString) {
    assert(pString.mLength > strlen(pString.mChars));
    mLength = pString.mLength;
    mChars = new char[mLength];
    strcpy(mChars, pString.mChars);
}

bool GString::operator==(const GString& pString) const {
    // check identity first:
    if (this == &pString) return(true);
    return(strcmp(mChars, (char*)pString) ? false : true);
}

bool GString::operator==(const char* pString) const {
    // check identity first:
    return(strcmp(mChars, pString) ? false : true);
}

GString& GString::operator=(const GString& pString) {
    int tLength = strlen(pString.mChars) + 1;
    if (tLength > mLength)  {
	mLength = tLength + tLength % 16;
	char* tChars = mChars;
	mChars = new char[mLength];
	strcpy(mChars, tChars);
	delete tChars;
    }
    strcpy(mChars, pString.mChars);
    assert(mLength > getLength());
    return(self);
}

GString GString::substr(int from, int to) {
    GString tStr(self);
    if (from > 0) tStr.del(0, from - 1);
    tStr.del(to + 1, tStr.getLength() - 1);
    return(tStr);
}

void GString::del(int from, int to) {
    if (from >= getLength()) return;
    if (to >= getLength()) to = getLength() - 1;
    strcpy(mChars + from, mChars + to + 1);
    assert(mLength > getLength());
}
