//////////////////////////////////////////////////////////////////////////////
//  This file belongs to the GENERIC 3D kernel. Copying, distribution and     
//  legal info is in the file COPYRIGHT which should be distributed with this 
//  file. If COPYRGHT is not available or for more info please contact: 
//		ekki@prakinf.tu-ilmenau.de
/////////// (C) '95 Ekkehard 'Ekki' Beier - All rights reserved. /////////////
#ifdef GWITH_TCL

#include <malloc.h>
#include "util/gtclpars.h"

const int gTclNextParseEnum = 11;
const char* gTclHelp = "?";
Tcl_Interp* gTclTmpInterp = Tcl_CreateInterp();
GString gTclTmpString(1000);
Tcl_Interp* gGlobalInterp = 0;

int gTclFindString(char** argv, const char *str) {
    int i = 0;
    while (*argv) {
	if (!strcmp(*argv, str)) return(i);
	argv++; 
	i++;
    }
    return(-1); 
}

void gTclClearArgv(char **argv, int start, int diff) {
    if (start >= 0) 
	for (int i = start; argv[i]; i++ ) 
	    argv[i] = argv[i + diff + 1];
}

void gTclParseTable(char** argv, GTclParseEntry* entry) {
    if (argv[0] && !strcmp(argv[0], gTclHelp)) {
	while (entry->mType != gTclEnd) {
	    if (entry->mIsSet) *entry->mIsSet = 0; 
	    // reset the entry flags
	    if (entry->mName[0] != '_') {
		// Do not print out hidden methods.
		sprintf(gTclTmpString, "{%s {%s} {%s}}\n", entry->mName, entry->mTclTypes, entry->mInfo ); 
		gTclAppendResult(gTclTmpString);
	    }
	    entry++;
	}
	return;
    }
    while (entry->mType != gTclEnd) {
	if (entry->mIsSet) *entry->mIsSet = 0;
	int start, diff;
	if ((entry->mType != gTclSpecial) && (start = gTclFindString(argv, entry->mName)) >= 0) {
	    diff = 0;
	    switch (entry->mType) {
	      case gTclNone:
		*entry->mIsSet = true;
		break;
	      case gTclString: 
		const char* fnd;
		if (gTclGetString((const char**)&argv[start + 1], fnd, diff)) {
		    *(char**)entry->mParam = (char*)fnd; 
		    *entry->mIsSet = true;
		}
		break;
	      case gTclBool:
		bool b;
		if (gTclGetBool((const char**)&argv[start + 1], b, diff)) {
		    *(bool*)entry->mParam = b; 
		    *entry->mIsSet = true;
		}
		break;
	      case gTclInt:
		int i;
		if (gTclGetInt((const char**)&argv[start + 1], i, diff)) {
		    *(int*)entry->mParam = i; 
		    *entry->mIsSet = true;
		}
		break;
	      case gTclFloat:
		GFloat f;
		if (gTclGetFloat((const char**)&argv[start + 1], f, diff)) {
		    *(GFloat*)entry->mParam = f; 
		    *entry->mIsSet = true;
		}
		break;
	      case gTclVector2: {
		  GVector2 vc2;
		  if (gTclGetVector2((const char**)&argv[start + 1], vc2, diff) ) {
		      *(GVector2*)entry->mParam = vc2; 
		      *entry->mIsSet = true;
		  }
	      }
		break;
	      case gTclVector3: {
		  GVector3 vc3;
		  if (gTclGetVector3((const char**)&argv[start + 1], vc3, diff) ) {
		      *(GVector3*)entry->mParam = vc3; 
		      *entry->mIsSet = true;
		  }
	      }
		break;
	      case gTclMatrix4: {
		  GMatrix4 ma;
		  if (gTclGetMatrix4((const char**)&argv[start + 1], ma, diff) ) {
		      *(GMatrix4*)entry->mParam = ma; 
		      *entry->mIsSet = true;
		  }
	      }
		break;
	      // just to make the compiler happy:
	      case gTclSpecial: break;
	      case gTclEnd: break;
	    }
	    gTclClearArgv(argv, start, diff);
	}
	entry++;
    }
}

// The utility functions return true if successful:

bool gTclGetBool(const char** argv, bool& bn, int& nr) {
    nr = 0;
    if (*argv) {
	bool b;
	if (gTclString2Bool(*argv, b)) {
	    bn = b;
	    nr = 1;
	    return(true);
	}
	return(false);
    }
    return(false);
}

bool gTclGetInt(const char** argv, int& in, int& nr) {
    nr = 0;
    if (*argv) {
	int i;
	if (gTclString2Int(*argv, i)) {
	    in = i;
	    nr = 1;
	    return(true);
	}
	return(false);
    }
    return(false);
}

bool gTclGetFloat(const char** argv, GFloat& fl, int& nr) {
    nr = 0;
    if (*argv) {
	GFloat f;
	if (gTclString2Float(*argv, f)) {
	    fl = f;
	    nr = 1;
	    return(true);
	}
	return(false);
    }
    return(false);
}

bool gTclGetString(const char** argv, const char*& s, int& nr) {
    nr = 0;
    if (*argv) {
	s = argv[0];
	nr = 1;
	return(true);
    }
    return(false);
}

bool gTclGetVector2(const char** s, GVector2& v, int& nr) {
    nr = 0;
    if (!*s) return(false);
    if (gTclString2Vector2(*s, v)) {
	nr = 1;
	return(true);
    }
    else return(false);
}

bool gTclGetVector3(const char** s, GVector3& v, int& nr) {
    nr = 0;
    if (!*s) return(false);
    if (gTclString2Vector3(*s, v)) {
	nr = 1;
	return(true);
    }
    else return(false);
}

bool gTclGetMatrix4(const char** s, GMatrix4& m, int& nr) {
    nr = 0;
    if (!*s) return(false);
    if (gTclString2Matrix4(*s, m)) {
	nr = 1;
	return(true);
    }
    else return(false);
}

bool gTclString2Bool(const char* s, bool& br) {
    int i;
    if (sscanf(s, "%i", &i)) {
	br = i ? true : false;
	return(true);
    }
    return(false);  
}

bool gTclString2Int(const char* s, int& ir) {
    int i;
    if (sscanf(s, "%i", &i)) {
	ir = i;
	return(true);
    }
    return(false);  
}

bool gTclString2Float(const char* s, GFloat& fr) {
    GFloat f;
    if (sscanf(s, 
#ifdef GSINGLE_PRECISION
	       "%f",
#else
	       "%lf",
#endif
	       &f)) {
	fr = f;
	return(true);
    }
    return(false);
}

bool gTclString2Vector2(const char* s, GVector2& v) {
    int argc; char **argv;
    if (Tcl_SplitList(gTclTmpInterp, (char*)s, &argc, &argv) == TCL_OK) {
	bool ok = false;
	GFloat x, y;
	if (argc == 2) {
	    if (gTclString2Float(argv[0], x))
		if (gTclString2Float(argv[1], y)) {
			ok = true;
			v.setX(x);
			v.setY(y);
		    }
	}
	free((char*)argv);
	return(ok);
    }
    else return(false);
}

bool gTclString2Vector3(const char* s, GVector3& v) {
    int argc; char **argv;
    if (Tcl_SplitList(gTclTmpInterp, (char*)s, &argc, &argv) == TCL_OK) {
	bool ok = false;
	GFloat x, y, z;
	if (argc == 3) {
	    if (gTclString2Float(argv[0], x))
		if (gTclString2Float(argv[1], y))
		    if (gTclString2Float(argv[2], z)) {
			ok = true;
			v.setX(x);
			v.setY(y);
			v.setZ(z);
		    }
	}
	free((char*)argv);
	return(ok);
    }
    else return(false);
}

bool gTclString2Matrix4(const char*s, GMatrix4& m) {
    int argc; char **argv;
    if (Tcl_SplitList(gTclTmpInterp, (char*)s, &argc, &argv) == TCL_OK) {
	bool ok = false;
	if (argc == 16) {
	    for (int j = 0; j < 4; j++) 
		for (int i = 0; i < 4; i++) {
		    GFloat f;
		    ok = gTclString2Float(argv[j * 4 + i], f);
		    m.set(j, i, f);
		    if (!ok) break;
		}
	}
	free((char*)argv);
	return(ok);
    }
    else return(false);
}

bool gTclString2Vector2List(const char* s, GDLList<GPoint2>& poly) {
    int argc; char **argv;
    if (Tcl_SplitList(gTclTmpInterp, (char*)s, &argc, &argv) == TCL_OK) {
	bool ok = false;
	if (argc > 0) {
	    ok = true;
	    for (int i = 0; i < argc; i++) {
		GVector2 p;
		if (gTclString2Vector2(argv[i], p)) {
		    poly.append(new GVector2(p));
		}
		else break;
	    }
	}
	free((char*)argv);
	return(ok);
    }
    else return(false);
}

void gTclAppendResult(const char* pRes) {
    assert(gGlobalInterp);
    Tcl_AppendResult(gGlobalInterp, (char*)pRes, 0);
}

int gTclGlobalEval(const char* pExpr) {
    assert(gGlobalInterp);
    return(Tcl_GlobalEval(gGlobalInterp, (char*)pExpr)); 
}

const char* gTclGetResult(void) {
    assert(gGlobalInterp);
    return(gGlobalInterp->result);
}

const char* gTclBool2String(bool b) {
    return(b ? "1" : "0");
}

const char* gTclInt2String(int i) {
    static char sString[10];
    sprintf(sString, "%i", i);
    return(sString);
}

const char* gTclFloat2String(GFloat f) {
    static char sString[10];
    sprintf(sString, 
#ifdef GSINGLE_PRECISION
	    "%f",
#else
	    "%lf",
#endif
	    f);
    return(sString);
}

const char* gTclVector32String(const GVector3& v) {
    static GString sString(30);
    sString = "{";
    sString += gTclFloat2String(v.getX());
    sString += " ";
    sString += gTclFloat2String(v.getY());
    sString += " ";
    sString += gTclFloat2String(v.getZ());
    sString += "}";
    return(sString);
}

#endif
