#include <render/gpixmap.h>
#include <render/graycam.h>
#include <render/grayligh.h>
#include <util/gtempl.h>

int main(void) {
    const GColorRGB cPink(0.5, 0.0, 0.7);

    // initialize GENERIC:
    gInitialize();

    // create a pixmap display to be used for
    // ray-tracing output:
    GPixmapDisplay tPixmap("GENERIC", 200, 200);
    
    // create an aggregate representing a scene:
    GDLList<GRayTraceable> tScene(false);

    // put an object into the scene:
    tScene.append(new GRayTraceableSphere);

    // initialize global ray-tracing light source aggregate:
    gRayLights = new GDLList<GRayLight>(false);

    // create a point light:
    GRayPointLight tLight;
    tLight.setPosition(GPoint3(3, 3, 3));
    tLight.setColor(GColorRGB(1, 1, 0));

    // insert light:
    gRayLights->append(&tLight); 

    // create the renderer:
    GLookatRayTracer<GRayTraceable, GPixmapDisplay> tRenderer;
    tRenderer.setBackground(cPink);

    // create the output:
    tRenderer.initRealization(tPixmap);
    tRenderer.rendering(tScene);

    return(0);
}
